; 1802 binary loader
; code produced by Jonathan Mordosky Oct-Nov 2015
; code changes & interpretation contributed by Lee Hart and Chuck Yakym 
; their program sources available where this source was found
; code was provided as freely available for use
; edited by Herb Johnson Feb 11 2016
;
; code as given below, uses EF3 as serial input
; serial hardware may invert serial signal, if so
; then swap all "BN3" and "B3" instructions and reassemble
;
; hex dump, TT is TPAGE value, PP is Page value, BN3 and B3 as in this source
;
;	0000	90 B4 A4 A5 F8 TT B5 E4 72 55 15 84 3A 08 F8 TT
;	0010	B0 3E 11 36 13 F8 PP B2 FE A2 E2 C4 C4 FC 01 3E
;	0020	1D B3 36 22 F8 FF 52 93 3E 28 F6 38 93 FF 01 3A
;	0030	2D 36 34 38 F6 02 76 52 C3 TT 2C 64 93 FF 01 3A
;	0040	3D 30 22
;
; If you swap BN3 and B3, the hex dump looks like this:
;
;	0000	90 B4 A4 A5 F8 TT B5 E4 72 55 15 84 3A 08 F8 TT
;	0010	B0 36 11 3E 13 F8 PP B2 FE A2 E2 C4 C4 FC 01 36
;	0020	1D B3 3E 22 F8 FF 52 93 36 28 F6 38 93 FF 01 3A
;	0030	2D 3E 34 38 F6 02 76 52 C3 TT 2C 64 93 FF 01 3A
;	0040	3D 30 22
;

	ORG	0000H	;memory where this program is toggled into

TPAGE	EQU	9CH	;memory page where this program is transferred
Page	EQU	80H	;memory page that bin loader, loads into

; this loop transfers loader into higher memory
INIT	GHI 0
	PHI 4
	PLO 4		; R4 = 0000H
	PLO 5
	LDI	TPAGE
	PHI 5		; R5 = 9C00H
	SEX 4		; X = 4
TRANFER LDXA
	STR 5
	INC 5
	GLO 4
	BNZ	TRANFER	; TRANFER THE WHOLE 1ST 256 BYTE BLOCK TO TPAGE ADDRESS
	LDI	TPAGE
	PHI 0		;program jumps to TPAGE block to run relocated loader below
			;low-order byte of R0 already points to next instruction below

;binary loader 
; code below has one long jump, so it can run from any memory page
; except the long-branch has to be adjusted as shown

			;A. wait for Start bit
LoopA	BN3  LoopA	;   loop while idle (EF3 false, EF3 pin high)
			;    continue when Start bit detected
			;   (EF3 true, EF3 pin goes low)

			;B. wait for Data bit D5=1 to begin
LoopB	B3 LoopB	;   loop while EF3 true (EF3 pin low)
					;   continue when EF3 false (pin goes high)

	LDI Page	;1  set R2 = destination Page address
	PHI 2		;2    high byte = Page 80H
	SHL		;3    clear low byte 	
	PLO 2		;4    low byte = 00 (and sets D=0)
	SEX 2		;5  set X=2 to point to destination
	NOP		;6.5 2 NOPs for 3 instruction times
	NOP		;8   (best detection margin at 9600 baud)

					;   measure remaining time in Data bit D5
Time	ADI 1		;9  D=D+1 for each loop
	BN3  Time	;10 ...loop until end of bit
	PHI 3		;C. found end, save Delay in R3.1

;to debug baud rate, uncomment following code & reassemble
;	STR 2		; option to display Delay and loop
;	OUT 4		; (use to test baud rate auto-detection)
;	BR 0

; OK, we're ready to receive bytes

Main	B3 Main	;D. wait for Stop Bit (EF3 pin low)
					;   found it, get ready for next byte
	LDI 0FFH	;     initialize byte to FF (all 1's)
	STR 2		;     and store it in RAM
	GHI 3		;   get Delay

LoopC	BN3  LoopC	;A. wait for Start bit (EF3 pin high)
					;   aha, found it

	SHR		;   Delay/2 to wait 1/2 bit (middle of bit)
	SKP		;   skip to Delay to read next bit
				;   (ideal is 3.5+Delay, this is 3+Delay)
; read next bit

NextBit			;   delay one bit-time
	GHI 3		;1   get Delay constant (skipped on first entry)
Delay	SMI 1		;2   decrement Delay...
	BNZ Delay	;3   loop until 0 (leaves D=00, DF=1)

	B3 Zero	;4  test next bit
	SKP		;5   if bit=1 (EF3 pin high), leave DF=1
Zero	SHR		;5   if bit=0 (EF3 pin low), set DF=0
	LDN 2		;6  get byte
	SHRC		;7   shift bit into byte (DF into D7, D0 into DF)
	STR 2		;8   store byte in RAM

;	LBDF NextBit ;9.5 if DF=1, then Start bit hasn't shifted all
			;    the way thru byte. Loop until it does
; to force assembler to create LBDF long-branch for *relocated loader*
	DB	0C3H		;LBDF opcode
	DB	TPAGE		;points to relocated code 
	DB	LOW(NextBit) ;load-byte addr of NextBit

	OUT 4		;   show byte and INC x-Pointer

			; added delay between last bit and stop bit
	GHI 3		;1   get Delay constant (skipped on first entry)
Delay1 SMI 1	;2   decrement Delay...
	BNZ Delay1	;3   loop until 0 (leaves D=00, DF=1)

	BR Main	;   get next byte
	END
